/*
 * CarManagerMIDlet.java
 *
 * Car Manager
 *
 * Copyright (C) 2005 Tommi Laukkanen
 * http://www.substanceofcode.com
 *
 * Created on June 6th 2005, 19:52
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package com.substanceofcode.carmanager.controller;

import com.substanceofcode.carmanager.model.CarManagerSettings;
import com.substanceofcode.carmanager.model.FileExporter;
import com.substanceofcode.carmanager.model.FillupEntryParser;
import com.substanceofcode.carmanager.view.SettingsForm;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.*;
import java.lang.Long;
import com.substanceofcode.carmanager.view.EntryForm;
import com.substanceofcode.carmanager.view.ExportForm;
import com.substanceofcode.carmanager.view.ReportForm;
import com.substanceofcode.carmanager.view.SplashCanvas;
import com.substanceofcode.carmanager.view.AboutForm;
import com.substanceofcode.carmanager.view.TimeSpanForm;
import com.substanceofcode.carmanager.model.CarManagerDatabase;
import com.substanceofcode.carmanager.model.FillupEntry;
import com.substanceofcode.carmanager.view.WaitDatabaseForm;

/**
 * Car Manager MIDlet
 *
 * @author  Tommi Laukkanen
 * @version 1.2
 */
public class CarManagerMIDlet extends MIDlet implements CommandListener {
    
    private Display display;		// The display for this MIDlet
    
    /** Commands */
    private Command m_exitCommand;
    private Command m_cmdAddGas =   new Command("Add", Command.SCREEN, 3);
    private Command m_cmdDelGas =   new Command("Remove", Command.SCREEN, 3);
    private Command m_cmdEditGas =  new Command("Edit", Command.SCREEN, 4);
    private Command m_cmdReport =   new Command("Report", Command.SCREEN, 5);
    private Command m_cmdSettings = new Command("Settings", Command.SCREEN, 6);
    private Command m_cmdAbout =    new Command("About", Command.SCREEN, 7);
    private Command m_cmdEraseAll = new Command("Remove all", Command.SCREEN, 8);
    private Command m_cmdExport =   new Command("Export", Command.SCREEN, 7);
    
    /** Forms and views */
    private EntryForm        m_entryForm; // = new EntryForm(this);
    private WaitDatabaseForm m_waitDatabaseForm = null;
    private SplashCanvas     m_splashCanvas = null;
    private ReportForm       m_reportForm = null;
    private AboutForm        m_viewAbout = null;
    private ExportForm       m_exportForm;
    private TimeSpanForm     m_timeSpanForm = null;
    private Thread           m_openDatabaseThread;
    private SettingsForm     m_settingsForm;
    
    /** Settings */
    private CarManagerSettings m_settings;
    CarManagerDatabase carDatabase = new CarManagerDatabase();
    MainCanvas m_mainCanvas;
    Vector fillupEntries;
    int selection;
    private int first;
    private long m_lastKilometers = 0;
    
    public CarManagerMIDlet() {
        m_mainCanvas = new MainCanvas();
        fillupEntries = new Vector();
        display = Display.getDisplay(this);
        m_exitCommand = new Command("Exit", Command.SCREEN, 5);
        first = 0;
        m_splashCanvas = new SplashCanvas( this );
        
    }
    
    /**
     * Start up the Car Manager MIDlet by creating the database connection
     * and associating different commands and listener to main canvas.
     */
    public void startApp(){
        
        m_mainCanvas.addCommand(m_exitCommand);
        m_mainCanvas.addCommand(m_cmdAddGas);
        m_mainCanvas.addCommand(m_cmdEditGas);
        m_mainCanvas.addCommand(m_cmdDelGas);
        m_mainCanvas.addCommand(m_cmdReport);
        m_mainCanvas.addCommand(m_cmdSettings);
        m_mainCanvas.addCommand(m_cmdEraseAll);
        m_mainCanvas.addCommand(m_cmdAbout);
        m_mainCanvas.addCommand(m_cmdExport);
        m_mainCanvas.setCommandListener(this);
        display.setCurrent( m_splashCanvas );
    }
    
    /** Open up the database */
    public void openDatabase() {
        try {
            carDatabase.open("CarManager");
        } catch (Exception e) {
            try {
                carDatabase.cleanUp("CarManager");
            } catch (Exception e2) {
            }
        }
        refreshView();
        showMainCanvas();
    }
    
    /** Open settings */
    public void openSettings() {
        m_settings = CarManagerSettings.getInstance(this);
    }
    
    /** Close database on pause */
    public void pauseApp() {
        try {
            carDatabase.close();
            carDatabase = null;
        } catch (Exception e) {}
    }
    
    /**
     * Destroy must cleanup everything not handled by the garbage collector.
     * In this case there is nothing but database to cleanup.
     */
    public void destroyApp(boolean unconditional) {
        try {
            carDatabase.close();
        } catch (Exception e) {}
    }
    
    /*
     * Respond to commands, including exit
     * On the exit command, cleanup and notify that the MIDlet has been destroyed.
     */
    public void commandAction(Command c, Displayable s) {
        if(c==m_exitCommand) {
            destroyApp(false);
            notifyDestroyed();
            
        } else if(c==m_cmdDelGas) {
            // Delete selected item
            try {
                FillupEntry del = (FillupEntry)(fillupEntries.elementAt( selection ));
                System.out.println("Del data: " + del.getOriginalRecord());
                carDatabase.delete( del.getOriginalRecord() );
                fillupEntries.removeElementAt( selection );
                if( selection > fillupEntries.size()-1 )
                    if( selection>0 )
                        selection--;
            } catch(Exception e) {
                System.err.println(e.toString());
            }
            m_mainCanvas.repaint();
            
        }else if(c==m_cmdEraseAll) {
            // Clear the databaae
            try {
                carDatabase.close();
                carDatabase.cleanUp("CarManager");
                fillupEntries.removeAllElements();
            } catch(Exception e) {
                System.err.println("Tyhjennys: " + e.toString());
            }
            m_mainCanvas.repaint();
            
        } else if(c==m_cmdAddGas) {
            m_entryForm = getEntryForm();
            m_entryForm.clearView(m_lastKilometers + 600);
            display.setCurrent(m_entryForm);
            
        } else if(c==m_cmdEditGas) {
            m_entryForm = getEntryForm();
            m_entryForm.setView((FillupEntry)fillupEntries.elementAt( selection ) );
            display.setCurrent(m_entryForm);
            
        } else if(c==m_cmdSettings) {
            showSettings();
            
        } else if(c==m_cmdReport) {
            showReport();
            
        } else if(c==m_cmdAbout) {
            showAboutForm();
            
        } else if (c == m_cmdExport) {
            showExportForm();
        }
    }
    
    public void showSettings() {
        if( m_settingsForm==null ) {
            /** Initialize settings form */
            m_settingsForm = new SettingsForm( this );
            
            /** Initialize form with current settings values */
            int entryType = m_settings.getEntryType();
            m_settingsForm.setSettings(entryType);
        }
        display.setCurrent(m_settingsForm);
    }
    
    public void showAboutForm() {
        if( m_viewAbout==null ) {
            m_viewAbout = new AboutForm( this );
        }
        display.setCurrent(m_viewAbout);
    }
    
    /** Show empty entry form */
    public void showEntryForm() {
        m_entryForm = getEntryForm();
        display.setCurrent(m_entryForm);
    }
    
    public void showExportForm() {
        if( m_exportForm==null) {
            m_exportForm = new ExportForm( this );
        }
        display.setCurrent(m_exportForm);
    }
    
    /** Show filled entry form */
    public void showEntryForm(FillupEntry entry) {
        m_entryForm = getEntryForm();
        m_entryForm.setView(entry);
        display.setCurrent(m_entryForm);
    }
    
    /** Get report start date */
    public Date getReportStartDate() {
        if( m_timeSpanForm==null ) {
            m_timeSpanForm = new TimeSpanForm(this);
        }
        return m_timeSpanForm.getStartDate();
    }
    
    /** Get report end date */
    public Date getReportEndtDate() {
        if( m_timeSpanForm==null ) {
            m_timeSpanForm = new TimeSpanForm(this);
        }
        return m_timeSpanForm.getEndDate();
    }
    
    /** Update old fill up entry */
    public void updateFillUp(
            FillupEntry tank,
            String newRecord,
            String oldRecord) {
        System.out.println("Update old record");
        System.out.println("New record: " + newRecord);
        System.out.println("Old record: " + oldRecord);
        try {
            carDatabase.update( oldRecord, newRecord.getBytes()  );
        } catch (RecordStoreNotOpenException ex) {
            ex.printStackTrace();
        } catch (RecordStoreFullException ex) {
            ex.printStackTrace();
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        }
        fillupEntries.setElementAt(tank, selection);
        fillupEntries = sortFillUps(fillupEntries);
    }
    
    /** Add new fill up entry */
    public void addFillUp(FillupEntry entry, String newRecord) {
        // Add to database
        try {
            System.out.println("Adding new entry");
            carDatabase.add( newRecord );
            fillupEntries.addElement(entry);
        }  catch (Exception e) {
            System.err.println("Add new entry: " + e.toString());
        }
        fillupEntries = sortFillUps(fillupEntries);
    }
    
    /** Get fill-ups */
    public Vector getFillups() {
        return fillupEntries;
    }
    
    /** Show fill-ups list */
    public void showFillups() {
        display.setCurrent( m_mainCanvas );
    }
    
    /** Show wait database form */
    public void showWaitDatabaseForm() {
        if( m_waitDatabaseForm==null ) {
            m_waitDatabaseForm = new WaitDatabaseForm(this);
        }
        display.setCurrent( m_waitDatabaseForm );
        m_openDatabaseThread = new Thread() {
            public void run() {
                System.out.println("Thread running");
                openDatabase();
                openSettings();
                System.out.println("Thread terminating");
            }
        };
        m_openDatabaseThread.start();
    }
    
    /** Show time span form */
    public void showTimeSpan() {
        if( m_timeSpanForm==null ) {
            m_timeSpanForm = new TimeSpanForm(this);
        }
        display.setCurrent( m_timeSpanForm );
    }
    
    /** Show report form */
    public void showReport() {
        if( m_reportForm==null ) {
            m_reportForm = new ReportForm(this);
        }
        m_reportForm.refresh();
        display.setCurrent( m_reportForm );
    }
    
    /*
     * Refresh the main list
     */
    public void refreshView(){
        try {
            // Just return if the database is empty
            if (carDatabase.getNumRecords() == 0) {
                return;
            }
            
            // Get all the records
            RecordEnumeration re = carDatabase.enumerateRecords();
            while (re.hasNextElement()) {
                String entryString = new String(re.nextRecord());
                System.out.println("Tankkaus: " + entryString);
                FillupEntry tank = FillupEntryParser.parseEntry( entryString );
                fillupEntries.addElement(tank);
                if( tank.getKm()>m_lastKilometers)
                    m_lastKilometers = tank.getKm();
            }
            fillupEntries = sortFillUps(fillupEntries);
        } catch (Exception e) {
            System.err.println("Refresh: " + e.toString());
        }
    }
    
    public String Cents2Normal(String number) {
        String cents = "00";
        String normal = "0";
        int len;
        len = number.length();
        if( len>2 ) {
            cents = number.substring(len-2, len);
            normal = number.substring(0, len-2);
        }
        if( len==2 ) {
            cents = number;
        }
        if( len==1 ) {
            cents = "0" + number;
        }
        return normal + "." + cents;
    }
    
    /** Use bubble sort to sort fill up entries */
    public Vector sortFillUps(Vector v) {
        FillupEntry first, second;
        for( int i=0; i<v.size(); i++) {
            for( int j=0; j<v.size(); j++) {
                first = (FillupEntry)v.elementAt(j);
                second = (FillupEntry)v.elementAt(i);
                
                long lngFirst = first.getKm();
                long lngSecond = second.getKm();
                if( lngFirst < lngSecond ) {
                    v.setElementAt(second,j);
                    v.setElementAt(first,i);
                }
            }
        }
        return v;
    }
    
    public void showMainCanvas() {
        display.setCurrent(m_mainCanvas);
    }
    
    public EntryForm getEntryForm() {
        m_entryForm = new EntryForm(this, m_settings.getEntryType());
        return m_entryForm;
    }

    /** Export fill-up entries to CSV file */
    public boolean exportFillUps(String path) {
        FileExporter exporter = new FileExporter();
        try {
            exporter.export( fillupEntries, path );
            return true;
        } catch(Exception ex) {
            return false;
        }        
    }
    
    class MainCanvas extends Canvas implements CommandListener {
        private int width;	// Ruudun leveys
        private int height;	// Ruudun korkeus
        private Image imgWrench;
        private Image imgTank;
        private int intFirst;	// First record in view
        private int intMaxInView; // Maximum records in view
        
        public MainCanvas() {
            intFirst = 0;
            width = getWidth();
            height = getHeight();
            selection = 0;
            repaint(0, 0, getWidth(), getHeight());
            imgWrench = loadImage("/icons/wrench.png");
            imgTank = loadImage("/icons/tank.png");
        }
        
        public void paint(Graphics g) {
            // Draw the fill up list
            g.setColor(255, 255, 255);
            g.fillRect(0, 0, width, height);
            paintList(g);
        }
        
        private void paintList(Graphics g) {
            String strTitle = "Fill-ups";
            
            g.setFont(Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_SMALL)); // Font.SIZE_SMALL or Font.SIZE_LARGE
            //                          FACE_SYSTEM       STYLE_PLAIN
            g.setColor(0, 0, 0);
            Font f = g.getFont();
            int intTitleHeight = f.getHeight();
            int intDateWidth = f.stringWidth("88/88") + 2;
            int intLitresWidth = f.stringWidth("88.88") + 2;
            int intPriceWidth = f.stringWidth("88.88") + 2;
            int intKmWidth = f.stringWidth("888888") + 2;
            int intInfoWidth = f.stringWidth("AAAAA") + 2;
            
            if (intDateWidth + intLitresWidth + intPriceWidth > width - 20) {
                intKmWidth = 0;
            }
            if (intKmWidth>0 && intDateWidth + intLitresWidth + intPriceWidth + intKmWidth > width - intInfoWidth) {
                intInfoWidth = 0;
            }
            
            if( selection < intFirst ) {
                intFirst--;
            }
            
            /** Draw selection bar as light blue */
            if( fillupEntries.size()>0 ) {
                g.setColor(200, 200, 255);
                g.fillRect(0, (selection - intFirst) * intTitleHeight +
                        intTitleHeight * 2 + 3, width-1, intTitleHeight );
                g.setColor(100, 100, 155);
                g.drawRect(0, (selection - intFirst) * intTitleHeight +
                        intTitleHeight * 2 + 2, width-1, intTitleHeight+1 );
            }
            
            g.setColor(0, 0, 0);
            
            g.drawImage(imgTank, 1, 1, g.TOP | g.LEFT);	// FillupEntry image
            g.drawString(strTitle, 12,1,  g.TOP | g.LEFT);
            g.drawLine(0,intTitleHeight, width, intTitleHeight);
            g.drawLine(intDateWidth,intTitleHeight, intDateWidth, height);
            g.drawLine(intDateWidth + intLitresWidth,intTitleHeight, intDateWidth + intLitresWidth, height);
            if(intKmWidth>0)
                g.drawLine(intDateWidth + intLitresWidth + intPriceWidth,intTitleHeight, intDateWidth + intLitresWidth + intPriceWidth, height);
            if(intInfoWidth>0)
                g.drawLine(intDateWidth + intLitresWidth + intPriceWidth + intKmWidth,intTitleHeight, intDateWidth + intLitresWidth + intPriceWidth + intKmWidth, height);
            g.drawString("date", 2,intTitleHeight+2,  g.TOP | g.LEFT);
            g.drawString("gas", intDateWidth+2,intTitleHeight+2,  g.TOP | g.LEFT);
            g.drawString("", intDateWidth + intLitresWidth + 4,intTitleHeight+2,  g.TOP | g.LEFT);
            if(intKmWidth>0) {
                g.drawString("km", intDateWidth + intLitresWidth + intPriceWidth + 6,intTitleHeight+2,  g.TOP | g.LEFT);
            }
            if(intInfoWidth>0) {
                g.drawString("info", intDateWidth + intLitresWidth + intPriceWidth + intKmWidth + 6,intTitleHeight+2,  g.TOP | g.LEFT);
            }
            g.drawLine(0,intTitleHeight * 2 + 1, width, intTitleHeight * 2 + 1);
            g.drawLine(0,height -1, width, height-1);
            
            int intRow = 0;
            g.setFont(Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL)); // Font.SIZE_SMALL or Font.SIZE_LARGE
            f = g.getFont();
            intTitleHeight = f.getHeight();
            
            /** Draw everything else as black */
            g.setColor(0, 0, 0);
            
            int intDrawnItems = 0;
            Enumeration IDs = fillupEntries.elements();
            while (IDs.hasMoreElements()) {
                FillupEntry tank = (FillupEntry) IDs.nextElement();
                if( intRow >= intFirst ) {
                    int y = intDrawnItems*intTitleHeight + intTitleHeight * 2 + 4;
                    if( y>height-(intTitleHeight+2)) break;
                    String month = tank.getDate();
                    month = month.substring(4, 6);
                    String day = tank.getDate().substring(6, 8);
                    String gas = Cents2Normal(Integer.toString(tank.getGas()));
                    String price = Cents2Normal(Integer.toString(tank.getPrice()));
                    long km = tank.getKm();
                    g.drawString( day + "/" + month,    2,                                y,  g.TOP | g.LEFT);
                    g.drawString( gas,   				intDateWidth+2,                   y,  g.TOP | g.LEFT);
                    g.drawString( price, 				intDateWidth + intLitresWidth + 2,y,  g.TOP | g.LEFT);
                    if(intKmWidth>0)
                        g.drawString( Long.toString(km), intDateWidth + intLitresWidth + intPriceWidth + 2,y,  g.TOP | g.LEFT);
                    if(intInfoWidth>0)
                        g.drawString( tank.getComment(), intDateWidth + intLitresWidth + intPriceWidth + intKmWidth + 2,y,  g.TOP | g.LEFT);
                    intDrawnItems++;
                }
                intRow++;
            }
            intMaxInView = intDrawnItems; // Number of items in the view
            g.setColor(100, 100, 100);
            
            if( intFirst>0 ) {
                g.drawLine(width-10, intTitleHeight / 2, width-6, intTitleHeight / 2 - 4 );
                g.drawLine(width-2,  intTitleHeight / 2, width-6, intTitleHeight / 2 - 4 );
            }
            if( intFirst + intDrawnItems < fillupEntries.size() ) {
                g.drawLine(width-10, intTitleHeight / 2, width-6, intTitleHeight / 2 + 4 );
                g.drawLine(width-2,  intTitleHeight / 2, width-6, intTitleHeight / 2 + 4 );
            }
        }
        
        protected void keyPressed(int keyCode) {
            int gameAction = getGameAction(keyCode);
            
            switch(gameAction) {
                case FIRE:
                    /** Display selected entry */
                    FillupEntry entry;
                    entry = (FillupEntry)fillupEntries.elementAt( selection );
                    showEntryForm( entry );
                    break;
                case RIGHT:
                    break;
                case DOWN:
                    selection ++;
                    if( selection>fillupEntries.size()-1 ) selection = fillupEntries.size()-1;
                    if( selection>=(intMaxInView+intFirst) ) {
                        intFirst++;
                    }
                    break;
                case LEFT:
                    break;
                case UP:
                    selection --;
                    if( selection<0 ) selection = 0;
                    break;
                default:
                    break;
            }
            repaint();
        }
        
        public void commandAction(Command c, Displayable d) {
            repaint();
        }
        
        private Image loadImage(String filename) {
            System.out.println("Loading image: " + filename);
            Image image = null;
            try {
                image = Image.createImage(filename);
            } catch(Exception e) {
                System.out.println("Error while loading image: " + filename);
                System.out.println("Description: " + e.toString());
                // Use null
            }
            return image;
        }
    }
    
}


